from pathlib import Path

from patent_client.session import PatentClientSession


class PtabSession(PatentClientSession):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.verify = str(Path(__file__).parent / "developer.uspto.gov.crt")



from patent_client.util import Manager

from .api import PublishedApi
from .cql import generate_query
from .schema import BiblioResultSchema
from .schema import ClaimsSchema
from .schema import DescriptionSchema
from .schema import ImagesSchema
from .schema import SearchSchema


class SearchManager(Manager):
    result_size = 100
    primary_key = "publication"
    __schema__ = SearchSchema
    __item_schema__ = BiblioResultSchema

    def __init__(self, config=None):
        super().__init__(config=None)
        if callable(self.__item_schema__):
            self.__item_schema__ = self.__item_schema__()

    def _get_search_results_range(self, start=1, end=100):
        if "cql_query" in self.config.filter:
            query = self.config.filter["cql_query"]
        else:
            query = generate_query(**self.config.filter)
        return self.__schema__.load(PublishedApi.search.search(query, start, end))

    def __len__(self):
        page = self._get_search_results_range(1, 100)
        offset = self.config.offset or 0
        limit = self.config.limit or page.num_results - offset
        num_results = page.num_results
        num_results -= offset
        num_results = min(limit, num_results)
        return num_results

    def _get_results(self):
        if len(self) == 0:
            return
        num_pages = round(len(self) / self.result_size)
        limit = self.config.limit or len(self)
        offset = self.config.offset or 0
        max_position = offset + limit
        range = (offset + 1, min(offset + self.result_size, max_position))
        while True:
            page = self._get_search_results_range(*range)
            for result in page.results:
                yield result
            if range[1] == max_position:
                break
            range = (
                range[0] + self.result_size,
                min(range[1] + self.result_size, max_position),
            )

    def get(self, number, doc_type="publication", format="docdb"):
        result = self.__item_schema__.load(PublishedApi.biblio.get_biblio(number, doc_type, format))
        if len(result.documents) > 1:
            raise Exception("More than one result found! Try another query")
        return result.documents[0]


class BiblioManager(Manager):
    __schema__ = BiblioResultSchema

    def get(self, doc_number):
        result = self.__schema__.load(PublishedApi.biblio.get_biblio(doc_number))
        if len(result.documents) > 1:
            raise ValueError(f"More than one result found for {doc_number}!")
        return result.documents[0]


class ClaimsManager(Manager):
    __schema__ = ClaimsSchema

    def get(self, doc_number):
        return self.__schema__.load(PublishedApi.fulltext.get_claims(doc_number))


class DescriptionManager(Manager):
    __schema__ = DescriptionSchema

    def get(self, doc_number):
        return self.__schema__.load(PublishedApi.fulltext.get_description(doc_number))


class ImageManager(Manager):
    __schema__ = ImagesSchema

    def get(self, doc_number):
        return self.__schema__.load(PublishedApi.images.get_images(doc_number))



from .api import convert_number


def test_docdb_to_epodoc():
    actual = convert_number("MD.20050130.A.20050130", "application", "docdb", "epodoc")
    assert not actual.messages
    assert actual.input_doc.id_type == "docdb"
    assert actual.output_doc.id_type == "epodoc"


def test_original_to_docdb():
    actual = convert_number("JP.(2006-147056).A.20060526", "application", "original", "docdb")
    assert not actual.messages
    assert actual.input_doc.id_type == "original"
    assert actual.output_doc.id_type == "docdb"


def test_docdb_to_original():
    actual = convert_number("JP.2006147056.A.20060526", "application", "docdb", "original")
    assert not actual.messages
    assert actual.input_doc.id_type == "docdb"
    assert actual.output_doc.id_type == "original"


def test_original_to_epodoc():
    actual = convert_number("US.(08/921,321).A.19970829", "application", "original", "epodoc")
    assert not actual.messages
    assert actual.input_doc.id_type == "original"
    assert actual.output_doc.id_type == "epodoc"


def test_original_to_docdb_pct():
    actual = convert_number("PCT/GB02/04635.20021011", "application", "original", "docdb")
    assert actual.input_doc.id_type == "original"
    assert actual.output_doc.id_type == "docdb"



from patent_client.util import Manager

from .api import FamilyApi
from .schema import FamilySchema


class FamilyManager(Manager):
    __schema__ = FamilySchema

    def get(self, doc_number):
        return self.__schema__.load(FamilyApi.get_family(doc_number, doc_type="publication", format="docdb"))



from pathlib import Path

from .api import PublishedApi

expected_dir = Path(__file__).parent / "test" / "expected"


def test_doc_example_biblio():
    result = PublishedApi.biblio.get_biblio("EP1000000.A1", format="epodoc")
    expected_file = expected_dir / "ep1000000_biblio_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected


def test_doc_example_full_cycle():
    result = PublishedApi.biblio.get_full_cycle("EP1000000.A1", format="epodoc")
    expected_file = expected_dir / "ep1000000_full_cycle_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected


def test_doc_example_abstract():
    result = PublishedApi.biblio.get_abstract("EP1000000.A1", format="epodoc")
    expected_file = expected_dir / "ep1000000_abstract_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected


def test_search():
    result = PublishedApi.search.search("ti=plastic")
    expected_file = expected_dir / "search_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected


def test_description():
    result = PublishedApi.fulltext.get_description("EP1000000.A1", format="epodoc")
    expected_file = expected_dir / "ep1000000_description_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected


def test_claims():
    result = PublishedApi.fulltext.get_claims("EP1000000.A1", format="epodoc")
    expected_file = expected_dir / "ep1000000_claims_result.xml"
    # expected_file.write_text(result)
    expected = expected_file.read_text()
    assert result == expected



import math
from typing import Generic

import inflection
from patent_client.util import Manager
from patent_client.util import ModelType

from . import schema_doc
from . import session
from .model import PtabDecision
from .model import PtabDocument
from .model import PtabProceeding
from .schema import PtabDecisionSchema
from .schema import PtabDocumentSchema
from .schema import PtabProceedingSchema
from .util import peds_to_ptab


class PtabManager(Manager, Generic[ModelType]):
    url = "https://developer.uspto.gov/ptab-api"
    page_size = 25
    instance_schema = None

    def _get_results(self):
        total = self._len()
        offset = self.config.offset
        limit = self.config.limit
        if limit:
            max_item = total if total - offset < limit else offset + limit
        else:
            max_item = total
        item_range = (offset, max_item)
        page_range = (
            int(offset / self.page_size),
            math.ceil(max_item / self.page_size),
        )
        counter = page_range[0] * self.page_size

        for p in range(*page_range):
            for item in self.get_page(p):
                if item_range[0] <= counter < item_range[1]:
                    yield self.__schema__.load(item)
                counter += 1
                if counter >= max_item:
                    return StopIteration

    def get_page(self, page_no):
        query = self.query()
        query["recordStartNumber"] = page_no * self.page_size
        response = session.get(self.url + self.path, params=query)
        return response.json()["results"]

    def __len__(self):
        length = self._len() - self.config.offset
        if self.config.limit:
            return length if length < self.config.limit else self.config.limit
        else:
            return length

    def _len(self):
        response = session.get(self.url + self.path, params=self.query())
        return response.json()["recordTotalQuantity"]

    def query(self):
        query = dict()
        for k, v in self.config.filter.items():
            key = k if k not in peds_to_ptab else peds_to_ptab[k]
            key = inflection.camelize(key, uppercase_first_letter=False)
            if isinstance(v, list):
                query[key] = " ".join(v)
            else:
                query[key] = v
        query["recordTotalQuantity"] = self.page_size
        query["sortOrderCategory"] = " ".join(
            inflection.camelize(o, uppercase_first_letter=False) for o in self.config.order_by
        )
        return query

    def allowed_filters(self):
        params = schema_doc["paths"][self.path]["get"]["parameters"]
        return {inflection.underscore(p["name"]): p["description"] for p in params}


class PtabProceedingManager(PtabManager[PtabProceeding]):
    path = "/proceedings"
    primary_key = "proceeding_number"
    __schema__ = PtabProceedingSchema()


class PtabDocumentManager(PtabManager[PtabDocument]):
    path = "/documents"
    primary_key = "document_identifier"
    __schema__ = PtabDocumentSchema()


class PtabDecisionManager(PtabManager[PtabDecision]):
    path = "/decisions"
    primary_key = "identifier"
    __schema__ = PtabDecisionSchema()



from pathlib import Path

from .api import LegalApi

test_dir = Path(__file__).parent / "test"
expected_dir = Path(__file__).parent / "test" / "expected"


def test_example():
    result = LegalApi.get_legal("EP1000000A1")
    expected_file = expected_dir / "example.xml"
    expected_file.write_text(result)
    expected = expected_file.read_text()
    assert expected == result



from patent_client.util.base.manager import Manager

from . import public_search_api
from .query import QueryBuilder
from .schema import PublicSearchDocumentSchema
from .schema import PublicSearchSchema


class CapacityException(Exception):
    pass


class PublicSearchManager(Manager):
    __schema__ = PublicSearchSchema
    page_size = 500
    primary_key = "patent_number"
    query_builder = QueryBuilder()

    def _get_results(self):
        query = self._query
        order_by = self._order_by
        sources = self.config.options.get("sources", ["US-PGPUB", "USPAT", "USOCR"])
        page_no = 0
        obj_counter = 0
        while True:
            page = public_search_api.run_query(
                query=query, start=page_no * self.page_size, limit=self.page_size, sort=order_by, sources=sources
            )
            for obj in page["patents"]:
                if self.config.limit and obj_counter >= self.config.limit:
                    break
                yield self.__schema__.load(obj)
                obj_counter += 1
            page_no += 1
            if len(page["patents"]) < self.page_size:
                break

    @property
    def _query(self):
        return self.query_builder.build_query(self.config)

    @property
    def _order_by(self):
        return self.query_builder.build_order_by(self.config)

    @property
    def query_fields(self):
        return self.query_builder.search_keywords

    @property
    def order_by_fields(self):
        return self.query_builder.order_by_keywords

    def __len__(self):
        if hasattr(self, "_len"):
            return self._len
        query = self._query
        order_by = self._order_by
        sources = self.config.options.get("sources", ["US-PGPUB", "USPAT", "USOCR"])
        page = public_search_api.run_query(query=query, start=0, limit=self.page_size, sort=order_by, sources=sources)

        total_results = page["totalResults"]
        total_results -= self.config.offset
        if self.config.limit:
            self._len = min(self.config.limit, total_results)
        else:
            self._len = total_results
        return self._len


class PublicSearchDocumentManager(PublicSearchManager):
    __doc_schema__ = PublicSearchDocumentSchema()

    def _get_results(self):
        result_count = super().__len__()
        if result_count > 20:
            raise CapacityException(
                f"Query would result in more than 20 results! ({result_count} > 20).\nPlease use the associated Biblio method to reduce load on the API (PublicSearch / PatentBiblio / PublishedApplicationBiblio"
            )

        for obj in super()._get_results():
            doc = public_search_api.get_document(obj)
            yield self.__doc_schema__.load(doc)


class PatentBiblioManager(PublicSearchManager):
    def __init__(self, config=None):
        super().__init__(config=config)
        self.config.options["sources"] = [
            "USPAT",
        ]


class PatentManager(PublicSearchDocumentManager):
    def __init__(self, config=None):
        super().__init__(config=config)
        self.config.options["sources"] = [
            "USPAT",
        ]


class PublishedApplicationBiblioManager(PublicSearchManager):
    def __init__(self, config=None):
        super().__init__(config=config)
        self.config.options["sources"] = [
            "US-PGPUB",
        ]


class PublishedApplicationManager(PublicSearchDocumentManager):
    def __init__(self, config=None):
        super().__init__(config=config)
        self.config.options["sources"] = [
            "US-PGPUB",
        ]



from patent_client.util import Manager

from .api import LegalApi
from .schema import LegalSchema


class LegalManager(Manager):
    __schema__ = LegalSchema

    def get(self, doc_number, doc_type="publication", format="docdb"):
        return self.__schema__.load(LegalApi.get_legal(doc_number, doc_type, format)).events



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['patent_client',
 'patent_client.epo',
 'patent_client.epo.ops',
 'patent_client.epo.ops.family',
 'patent_client.epo.ops.legal',
 'patent_client.epo.ops.number_service',
 'patent_client.epo.ops.published',
 'patent_client.epo.ops.published.model',
 'patent_client.epo.ops.published.schema',
 'patent_client.uspto',
 'patent_client.uspto.assignment',
 'patent_client.uspto.peds',
 'patent_client.uspto.ptab',
 'patent_client.uspto.public_search',
 'patent_client.util',
 'patent_client.util.base',
 'patent_client.util.claims',
 'patent_client.util.datetime']

package_data = \
{'': ['*'],
 'patent_client.epo.ops.family': ['fixtures/examples/*', 'fixtures/expected/*'],
 'patent_client.epo.ops.legal': ['test/*', 'test/expected/*'],
 'patent_client.epo.ops.published': ['test/*', 'test/expected/*'],
 'patent_client.uspto.assignment': ['test/*'],
 'patent_client.uspto.peds': ['test/*'],
 'patent_client.uspto.public_search': ['test/*'],
 'patent_client.util.claims': ['examples/*']}

install_requires = \
['PyPDF2>=2.2.0,<3.0.0',
 'PyYAML>=6.0,<7.0',
 'colorlog>=6.6.0,<7.0.0',
 'inflection>=0.5.1,<0.6.0',
 'lxml>=4.9.0,<5.0.0',
 'openpyxl>=3.0.10,<4.0.0',
 'pyparsing>=3.0.9,<4.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests-cache>=0.9.4,<0.10.0',
 'requests>=2.28.0,<3.0.0',
 'ujson>=5.4.0,<6.0.0',
 'yankee>=0.1.40,<0.2.0',
 'zipp>=3.8.1,<4.0.0']

extras_require = \
{'docs': ['furo>=2022.6,<2023.0',
          'linkify-it-py>=2.0,<3.0',
          'myst-parser>=0.17',
          'sphinx>=5.0.2,<6.0.0',
          'sphinx-autodoc-typehints>=1.19,<2.0',
          'sphinx-automodapi>=0.14',
          'sphinx-copybutton>=0.5',
          'sphinx-design>=0.2',
          'sphinx-notfound-page>=0.8',
          'sphinxcontrib-apidoc>=0.3,<0.4',
          'sphinxcontrib-mermaid>=0.7.1,<0.8.0',
          'nbsphinx>=0.8.9,<0.9.0',
          'IPython>=7.17.0,<8.0.0']}

setup_kwargs = {
    'name': 'patent-client',
    'version': '3.1.0',
    'description': 'A set of ORM-style clients for publicly available intellectual property data',
    'long_description': '[![patent_client_logo](https://raw.githubusercontent.com/parkerhancock/patent_client/master/docs/_static/patent_client_logo.svg)](https://patent-client.readthedocs.io)\n\n[![Build](https://github.com/parkerhancock/patent_client/actions/workflows/build.yaml/badge.svg)](https://github.com/parkerhancock/patent_client/actions/workflows/build.yaml)\n[![codecov](https://codecov.io/gh/parkerhancock/patent_client/branch/master/graph/badge.svg?token=pWsiQLHi6r)](https://codecov.io/gh/parkerhancock/patent_client)\n[![Documentation](https://img.shields.io/readthedocs/patent-client/stable)](https://patent-client.readthedocs.io/en/stable/)\n\n\n[![PyPI](https://img.shields.io/pypi/v/patent-client?color=blue)](https://pypi.org/project/patent-client)\n[![PyPI - Python Versions](https://img.shields.io/pypi/pyversions/patent-client)](https://pypi.org/project/patent-client)\n[![PyPI - Downloads](https://img.shields.io/pypi/dm/patent-client?color=blue)](https://pypi.org/project/patent-client)\n\n# Summary\n\nA powerful library for accessing intellectual property, featuring:\n\n- 🍰 **Ease of use:** All sources use a simple unified API inspired by [Django-ORM][DORM].\n- 🐼 **Pandas Integration:** Results are easily castable to [Pandas Dataframes and Series][PANDAS].\n- 🚀 **Performance:** Fetched data is cached using the excellent [requests-cache][requests-cache] library for super-fast queries.\n\nDocs, including a fulsome Getting Started and User Guide are available on [Read the Docs](http://patent-client.readthedocs.io). The Examples folder includes examples of using `patent_client` for\nmany common IP tasks\n\n## Coverage\n\n- [United States Patent & Trademark Office][USPTO]\n\n  - [Patent Public Search][PPS] - Full Support\n  - [Patent Examination Data][PEDS] - Full Support\n  - [Patent Assignment Data][Assignment] - Lookup Support\n  - [Patent Trial & Appeal Board API v2][PTAB] - Supports Proceedings, Decisions, and Documents\n\n\n- [European Patent Office - Open Patent Services][OPS]\n\n  - Inpadoc - Full Support\n  - EPO Register - No Support (in progress)\n  - Classification - No Support\n\n* Free software: Apache Software License 2.0\n\n[DORM]: https://docs.djangoproject.com/en/4.0/topics/db/queries/\n[PANDAS]: https://pandas.pydata.org/docs/\n[requests-cache]: https://github.com/requests-cache/requests-cache\n[Assignment]: https://developer.uspto.gov/api-catalog/patent-assignment-search-beta\n[OPS]: http://ops.epo.org\n[PPS]:  https://ppubs.uspto.gov/pubwebapp/static/pages/landing.html\n[PEDS]: https://developer.uspto.gov/api-catalog/ped\n[PTAB]: https://developer.uspto.gov/api-catalog/ptab-api-v2\n[USPTO]: http://developer.uspto.gov\n\n\n## Installation\n\n```\npip install patent_client\n```\n\nIf you only want access to USPTO resources, you\'re done!\nHowever, additional setup is necessary to access EPO Inpadoc and EPO Register resources. See the [Docs](http://patent-client.readthedocs.io).\n\n\n## Quick Start\n\nTo use the project:\n\n```python\n# Import the model classes you need\n>>> from patent_client import Inpadoc, Assignment, USApplication, PatentBiblio\n\n# Fetch US Patents with the word "tennis" in their title issued in 2010\n>>> pats = PatentBiblio.objects.filter(title="tennis", issue_date="2010-01-01->2010-12-31")\n>>> len(pats) > 10\nTrue\n\n# Look at the first one\n>>> pats[0]\nPublicationBiblio(publication_number=7841958, publication_date=2010-11-30, patent_title=Modular table tennis game)\n\n# Fetch US Applications\n>>> app = USApplication.objects.get(\'15710770\')\n>>> app.patent_title\n\'Camera Assembly with Concave-Shaped Front Face\'\n\n# Fetch from USPTO Assignments\n>>> assignments = Assignment.objects.filter(assignee=\'Google\')\n>>> len(assignments) > 23000\nTrue\n>>> assignment = Assignment.objects.get(\'47086-788\')\n>>> assignment.conveyance_text\n\'ASSIGNMENT OF ASSIGNORS INTEREST\'\n\n# Fetch from INPADOC\n>>> pub = Inpadoc.objects.get(\'EP3082535A1\')\n>>> pub.biblio.title\n\'AUTOMATIC FLUID DISPENSER\'\n\n```\n<!-- RTD-IGNORE -->\n\n## Documentation\n\nDocs, including a fulsome Getting Started are available on [Read the Docs](http://patent-client.readthedocs.io).\n\n<!-- END-RTD-IGNORE -->\n\n# Development\n\nTo run the all tests run:\n\n```\npytest\n```\n\nA developer guide is provided in the [Documentation](http://patent-client.readthedocs.io).\nPull requests welcome!\n\n# Related projects\n\n- [Python EPO OPS Client](https://github.com/55minutes/python-epo-ops-client)\n- [Google Public Patent Data](https://github.com/google/patents-public-data)\n- [PatentsView API Wrapper](https://github.com/mikeym88/PatentsView-API-Wrapper)\n- [USPTO Scrapy Scraper](https://github.com/blazers08/USPTO)\n\n',
    'author': 'Parker Hancock',
    'author_email': '633163+parkerhancock@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/parkerhancock/patent_client',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
